/*
 *  genrandom reads nbytes truly random bytes from /dev/random
 *  and writes them to the file "filename"
 *
 *  Version 0.1 by Andreas Steffen, Zuercher Hochschule Winterthur,
 *  Switzerland, January 4 2001
 */

#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>

int main(int argc, char *argv[])
{
	const char device[] = "/dev/random";
	/* const char device[] = "/dev/urandom" */	
	const int bufsize = 1024;
	char buf[bufsize];
	char *me;
	char *filename;
        int dev;
        int nbytes;
        int got;
        int put;
	FILE *f;

	/* read command line arguments */

        me = argv[0];

        if (argc != 3)
	{
		fprintf(stderr, "Usage:  %s nbytes filename\n",me);
		exit(1);
	}

	nbytes = atoi(argv[1]);
	filename = argv[2];

	/* open random device */

	dev = open(device, 0);

	if (dev < 0)
	{
		fprintf(stderr, "%s: Could not open %s (%s)\n",
					me, device, strerror(errno));
		exit(1);
	}

	/* open file "filename" for writing */

        f = fopen(filename,"w");

        if (f == NULL)
	{
		fprintf(stderr, "%s: unable to open file `%s' (%s)\n",
						me, filename, strerror(errno));
		exit(1);
	}

	/* read nbytes random bytes from random device and write it to file */

	while (nbytes > 0) {
		got = read(dev, buf, (nbytes < bufsize)?nbytes:bufsize);
		if (got < 0)
		{
			fprintf(stderr, "%s: read error on %s (%s)\n",
						me, device, strerror(errno));
			exit(1);
		}
		if (got == 0)
		{
			fprintf(stderr, "%s: eof on %s!?!\n", me, device);
			exit(1);
		}

		put = fwrite(buf, 1, got, f);

		if (put != got)
		{
			fprintf(stderr, "%s: write error on %s (%s)\n",
						me, filename, strerror(errno));
			exit(1);
		}
				
		nbytes -= got;
	}

	close(dev); fclose(f);
	exit(0);
}

